@@ -8,31 +8,30 @@
    ##################################################################### */
 									/*}}}*/
 // Include Files							/*{{{*/
-#include<config.h>
+#include <config.h>
 
+#include <apt-pkg/aptconfiguration.h>
 #include <apt-pkg/configuration.h>
-#include <apt-pkg/indexfile.h>
+#include <apt-pkg/deblistparser.h>
 #include <apt-pkg/error.h>
 #include <apt-pkg/fileutl.h>
-#include <apt-pkg/aptconfiguration.h>
+#include <apt-pkg/indexfile.h>
+#include <apt-pkg/macros.h>
 #include <apt-pkg/pkgcache.h>
 #include <apt-pkg/pkgcachegen.h>
-#include <apt-pkg/cacheiterators.h>
+#include <apt-pkg/progress.h>
 #include <apt-pkg/srcrecords.h>
 #include <apt-pkg/strutl.h>
-#include <apt-pkg/progress.h>
-#include <apt-pkg/deblistparser.h>
-#include <apt-pkg/macros.h>
 
 #include <apt-pkg/debindexfile.h>
 
 #include <sys/stat.h>
 
-#include <string>
-#include <vector>
 #include <clocale>
 #include <cstring>
 #include <memory>
+#include <string>
+#include <vector>
 									/*}}}*/
 
 // Global list of Item supported
@@ -137,16 +136,27 @@ std::string IndexTarget::Option(OptionKeys const EnumKey) const		/*{{{*/
       APT_CASE(ARCHITECTURE);
       APT_CASE(BASE_URI);
       APT_CASE(REPO_URI);
+      APT_CASE(IDENTIFIER);
       APT_CASE(TARGET_OF);
       APT_CASE(CREATED_BY);
+      APT_CASE(FALLBACK_OF);
       APT_CASE(PDIFFS);
       APT_CASE(DEFAULTENABLED);
       APT_CASE(COMPRESSIONTYPES);
       APT_CASE(SOURCESENTRY);
       APT_CASE(BY_HASH);
       APT_CASE(KEEPCOMPRESSEDAS);
+      APT_CASE(ALLOW_INSECURE);
+      APT_CASE(ALLOW_WEAK);
+      APT_CASE(ALLOW_DOWNGRADE_TO_INSECURE);
 #undef APT_CASE
-      case FILENAME: return _config->FindDir("Dir::State::lists") + URItoFileName(URI);
+      case FILENAME:
+      {
+	 auto const M = Options.find("FILENAME");
+	 if (M == Options.end())
+	    return _config->FindDir("Dir::State::lists") + URItoFileName(URI);
+	 return M->second;
+      }
       case EXISTING_FILENAME:
 	 std::string const filename = Option(FILENAME);
 	 std::vector<std::string> const types = VectorizeString(Option(COMPRESSIONTYPES), ' ');
@@ -168,7 +178,7 @@ std::string IndexTarget::Option(OptionKeys const EnumKey) const		/*{{{*/
 									/*}}}*/
 bool IndexTarget::OptionBool(OptionKeys const EnumKey) const		/*{{{*/
 {
-   return StringToBool(Option(EnumKey));
+   return StringToBool(Option(EnumKey), false);
 }
 									/*}}}*/
 std::string IndexTarget::Format(std::string format) const		/*{{{*/
@@ -327,16 +337,10 @@ pkgCacheListParser * pkgDebianIndexFile::CreateListParser(FileFd &Pkg)
    if (Pkg.IsOpen() == false)
       return nullptr;
    _error->PushToStack();
-   pkgCacheListParser * const Parser = new debListParser(&Pkg);
+   std::unique_ptr<pkgCacheListParser> Parser(new debListParser(&Pkg));
    bool const newError = _error->PendingError();
    _error->MergeWithStack();
-   if (newError)
-   {
-      delete Parser;
-      return nullptr;
-   }
-   else
-      return Parser;
+   return newError ? nullptr : Parser.release();
 }
 bool pkgDebianIndexFile::Merge(pkgCacheGenerator &Gen,OpProgress * const Prog)
 {
@@ -399,6 +403,6 @@ pkgCache::PkgFileIterator pkgDebianIndexFile::FindInCache(pkgCache &Cache) const
    return File;
 }
 
-APT_CONST pkgIndexFile::~pkgIndexFile() {}
-APT_CONST pkgDebianIndexTargetFile::~pkgDebianIndexTargetFile() {}
-APT_CONST pkgDebianIndexRealFile::~pkgDebianIndexRealFile() {}
+APT_PURE pkgIndexFile::~pkgIndexFile() {}
+APT_PURE pkgDebianIndexTargetFile::~pkgDebianIndexTargetFile() {}
+APT_PURE pkgDebianIndexRealFile::~pkgDebianIndexRealFile() {}
