@@ -1,10 +1,10 @@
 #include <config.h>
 
+#include <apt-pkg/aptconfiguration.h>
+#include <apt-pkg/configuration.h>
 #include <apt-pkg/error.h>
 #include <apt-pkg/fileutl.h>
 #include <apt-pkg/strutl.h>
-#include <apt-pkg/aptconfiguration.h>
-#include <apt-pkg/configuration.h>
 
 #include <algorithm>
 #include <string>
@@ -25,7 +25,9 @@ static void TestFileFd(mode_t const a_umask, mode_t const ExpectedFilePermission
 
    static const char* fname = "apt-filefd-test.txt";
    if (FileExists(fname) == true)
+   {
       EXPECT_EQ(0, unlink(fname));
+   }
 
    FileFd f;
    umask(a_umask);
@@ -150,6 +152,19 @@ static void TestFileFd(mode_t const a_umask, mode_t const ExpectedFilePermission
       EXPECT_EQ(strlen(expect), f.Tell());
    }
 #undef APT_INIT_READBACK
+   {
+      test.erase(test.length() - 1);
+      EXPECT_TRUE(f.Seek(0));
+      EXPECT_FALSE(f.Eof());
+      std::string line;
+      EXPECT_TRUE(f.ReadLine(line));
+      EXPECT_FALSE(f.Failed());
+      EXPECT_FALSE(f.Eof());
+      EXPECT_EQ(line.length(), test.length());
+      EXPECT_EQ(line.length() + 1, f.Tell());
+      EXPECT_EQ(f.Size(), f.Tell());
+      EXPECT_EQ(line, test);
+   }
 
    f.Close();
    EXPECT_FALSE(f.IsOpen());
@@ -290,7 +305,9 @@ TEST(FileUtlTest, Popen)
 
    // ensure that after a close all is good again
    if(FileExists("/proc/self/fd"))
+   {
       EXPECT_EQ(Glob("/proc/self/fd/*").size(), OpenFds.size());
+   }
 
    // ReadWrite is not supported
    _error->PushToStack();
